#include "curvature.h"
#include <iostream>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Copyright (c) 2013 by Nicolas Bonneel (nbonneel@seas.harvard.edu)
/// Test unit for curvature.h
/// Verifies that the curvature of a geodesic is (near) zero
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void curvature_test() {

	int N = 50;

	// let's use two curves
	std::vector<AffineCovarianceMatrix<2, double> > all_matrices(N);  
	std::vector<AffineCovarianceMatrix<2, double> > all_matrices2(N);

	// we define the two end points in these curves
	all_matrices[0][0] = 0.409392; 
	all_matrices[0][1] = 0.229454;
	all_matrices[0][2] = 0.229454;
	all_matrices[0][3] = 0.376641;
	all_matrices[0][4] = 0;
	all_matrices[0][5] = 0;
	all_matrices2[0][0] = 1.151758; 
	all_matrices2[0][1] = 0.834870;
	all_matrices2[0][2] = 0.834870;
	all_matrices2[0][3] = 0.660581;
	all_matrices2[0][4] = 0; 
	all_matrices2[0][5] = 0;
	
	all_matrices[N-1][0] = 1.484234*10.;  // some rotation + scaling 
	all_matrices[N-1][1] = 0.930788*10.;
	all_matrices[N-1][2] = 0.930788*10.;
	all_matrices[N-1][3] = 0.850381*10.;
	all_matrices[N-1][4] = 0.; // no translation for this one
	all_matrices[N-1][5] = 0.;

	all_matrices2[N-1][0] = 1.847829; // some rotation
	all_matrices2[N-1][1] = 1.087429;
	all_matrices2[N-1][2] = 1.087429;
	all_matrices2[N-1][3] = 0.966892;
	all_matrices2[N-1][4] = 10.; // some translation
	all_matrices2[N-1][5] = 10.;

	// compute the two geodesics using displacement interpolation 
	for (int i=1; i<N-1; i++) {
		double t = i/(double)(N-1);
	    all_matrices[i] = displacement_interpolation(all_matrices[0], 1.-t, all_matrices[N-1], t);
		all_matrices2[i] = displacement_interpolation(all_matrices2[0], 1.-t, all_matrices2[N-1], t);	
	}

	std::cout<<"the per-frame curvature of our "<<N<<" points geodesic consisting of 2 matrices per frame is :"<<std::endl;
	// now assume that at each time step, we form a point (M1, M2) where M1 is the matrix from the first curve (at time t) and M2 the matrix from the second curve (at time t)
	// the function below computes the curvature
	Curvature<AffineCovarianceMatrix<2, double> , 3, 2, AffineCovarianceMatrix<2, double> , 3, 2> curvature(&all_matrices2[0], &all_matrices[0], N, 1, 1);
	for (int i=0; i<N; i++) { // remove the boundaries as finite differences are only first order there
		double curv = curvature(i);
		std::cout<<curv<<std::endl;
	}
	
}

int main() {
	curvature_test();
	return 0;
}